#======================================================================
# Hungarian Multimetric Fish Index (HMMFI) for ecological assessment
# of running waters of Hungary
#
# HMMFI was developed by Péter SÁLY & Tibor ERŐS
#
# HMMFI Calculator R program was written by Péter Sály.
#
# Version 1.1
#
# Budapet, 2019-06-06
#======================================================================



#======================================================================
#           *** I. Preparations ***
#++++++++++++++++++++++++++++++++++++++++++++++++++
# Version number:
#++++++++++++++++
versnumb <- "1.1"
#++++++++++++++++++++++++++++++++++++++++++++++++++


#++++++++++++++++++++++++++++++++++++++++++++++++++
# Welcome MESSAGE:
#+++++++++++++++++
{cat("\n*** Start of program ***\n")
cat("\nWelcome! This is HMMFI calculator version ",versnumb,".\n", sep="")
cat("R code written by Péter SÁLY\n")
cat("2016-06-06\n")}
#++++++++++++++++++++++++++++++++++++++++++++++++++


#++++++++++++++++++++++++++++++++++++++++++++++++++
# Required libraries:
#++++++++++++++++++++
# Check:
#*******
# All necessary libraries installed
reqpack  <- c("tcltk")
instpack <- installed.packages()[,1:2]
instpack
a <- setdiff(reqpack, instpack)
if(length(a) !=0) {
    stop("\nYou need to install the following R package(s) to run:\n",a,"\n")}
rm(a)


# Load libraries:
#*****************
require(package="tcltk")
#++++++++++++++++++++++++++++++++++++++++++++++++++


#++++++++++++++++++++++++++++++++++++++++++++++++++
# Set working dir:
#++++++++++++++++++++
wd <- getwd()
setwd(wd)
#++++++++++++++++++++++++++++++++++++++++++++++++++


#++++++++++++++++++++++++++++++++++++++++++++++++++
# Read raw fish data:
#++++++++++++++++++++
# MESSAGE:
#*********
cat("\nPlease select the .csv file with your raw fish data to input from the pop-up window.\n\n")


# Input data table:
#******************
filter <- matrix(c("Comma separeted values", ".csv"), 1,2, byrow=T)
filetoread <- tk_choose.files(filters=filter,
                              caption ="Select your raw data file")
dat <- read.csv(file=filetoread, sep=";", header=TRUE,
                 fileEncoding="latin2",  skip=3, as.is=c(1:2))
##fix(dat)
##str(dat)
##head(dat)


# Check columns sampleID and type:
#*********************************
if(any(duplicated(dat[,1]))) {
    stop("\nIt seems your raw data table contains duplicated samples.\nCheck sample IDs.\n") }
if(any(is.na(dat[,2]))) {
    stop("\nMissing 'type'.\nCheck type classification of your samples.\n") }
if(!all(dat[,2] %in% 1:6)) {
    stop("\nWrong 'type'.\nCheck type classification of your samples.\n") }
#++++++++++++++++++++++++++++++++++++++++++++++++++


#++++++++++++++++++++++++++++++++++++++++++++++++++
# Read auxiliary data:
#++++++++++++++++++++++++
# Species trait table:
traittab <-
    read.table(file=paste(wd,"auxiliary_data/species_trait_table.csv", sep="/"), 
               sep=";", header=T, skip=2, as.is=c(1:4), fileEncoding="latin2")
##fix(traittab)
# Az angol, magyar, tudományos fajnevek leválasztása a traittábláról:
##as.matrix(colnames(traittab))
traittab2 <- traittab[,c(4:40)]

# traitdata() function:
##file.show(paste(wd,"auxiliary_data/traitdata_function_20150506_SP.R", sep="/"))
source(paste(wd,"auxiliary_data/traitdata_function_20150506_SP.R", sep="/"))

# Typespecific arguments and their values:
##file.show(paste(wd,"auxiliary_data/typespecific_argument_values.R", sep="/"))
source(paste(wd,"auxiliary_data/typespecific_argument_values.R", sep="/"))

# eqr() function:
##file.show(paste(wd,"auxiliary_data/EQR_function_20150629_SP.R", sep="/"))
source(paste(wd,"auxiliary_data/EQR_function_20150629_SP.R", sep="/"))
#++++++++++++++++++++++++++++++++++++++++++++++++++



#       *** II. Compliling trait data ***
#++++++++++++++++++++++++++++++++++++++++++++++++++
# Replace NAs with 0s in raw data table:
#***************************************
##head(dat)
dat <- replace(dat, is.na(dat), 0)
##head(dat)


# Creating trait tables:
#***********************
# Native and non-native species:
##as.matrix(colnames(traittab2))
# native species ("N"):      column  30
# non-native species ("NN"): column  31
##str(dat)
# Trait-relative abundance table:
suppressWarnings(
dat.trait.relab <- traitdata(x=dat[,-c(1:2)], traittab=traittab2,
                             method="relabund", nativeonly=T,
                             nonnativespec=31, nativespec=30) )
colnames(dat.trait.relab) <- paste(colnames(dat.trait.relab), "ra", sep=".")
dat.trait.relab           <- cbind(dat[,c(1:2)], dat.trait.relab)
##head(dat.trait.relab)

# Trait-species number table:
suppressWarnings(
dat.trait.specnumb <- traitdata(x=dat[,-c(1:2)], traittab=traittab2,
                                method="specnumb", nativeonly=T,
                                nonnativespec=31, nativespec=30) )
colnames(dat.trait.specnumb) <- paste(colnames(dat.trait.specnumb), "sn", sep=".")
dat.trait.specnumb           <- cbind(dat[,c(1:2)], dat.trait.specnumb)
##head(dat.trait.specnumb)

# Trait-relative species number table:
suppressWarnings(
dat.trait.relspecnumb <- traitdata(x=dat[,-c(1:2)], traittab=traittab2,
                                   method="relspecnumb", nativeonly=T,
                                   nonnativespec=31, nativespec=30) )
colnames(dat.trait.relspecnumb) <- paste(colnames(dat.trait.relspecnumb), "rsn", sep=".")
dat.trait.relspecnumb           <- cbind(dat[,c(1:2)], dat.trait.relspecnumb)
##head(dat.trait.relspecnumb)


# Merging trait tables:
#**********************
# Kapcsolómező: 'sampleID'
##as.matrix(colnames(dat.trait.relab))
##as.matrix(colnames(dat.trait.specnumb))
##as.matrix(colnames(dat.trait.relspecnumb))

a <- dat.trait.relab
b <- dat.trait.specnumb[,-2]
connector <- intersect(colnames(a), colnames(b))
connector # 'sampleID'
dat.trait.merged <- merge(x=a, y=b, by=connector,  sort=FALSE)
##dim(dat.trait.merged)
##head(dat.trait.merged)
rm(a,b)

a <- dat.trait.merged
b <- dat.trait.relspecnumb[,-2]
connector <- intersect(colnames(a), colnames(b))
connector # 'sampleID'
dat.trait.merged <- merge(x=a, y=b, by=connector,  sort=FALSE)
##dim(dat.trait.merged)
##head(dat.trait.merged)
rm(a,b)


# Replacing NaN-s with 0:
#************************
# This occurs when there are sample wih no fish at all.
dat.trait.merged <- replace(dat.trait.merged,is.na(dat.trait.merged),0)


# Character species pool analysis:
#*********************************
##as.matrix(colnames(traittab2))
##as.matrix(colnames(dat.trait.merged))
CSpecpool <- NULL
for(i in 1:nrow(dat.trait.merged)){
    # Set type:
    typ <- dat.trait.merged$type[i]
    # Total number of character species of the type:
    a <- sum(traittab2[,(31+typ)])
    # Number of character species present in the sample:
    b <- dat.trait.merged[i,(68+typ)]
    # CSpecpool value:
    CSpecpool[i] <- b/a
    # Remove a and b:
    rm(a,b)
} # End of for cycle i
CSpecpool
### Ellenőrzés:
##dat[5, traittab2[traittab$CSpec3==1,1]] # OK
# Merge CSpecpool and dat.trait.merged:
dat.trait <-  cbind(dat.trait.merged, CSpecpool)
##str(dat.trait[,-c(1:100)])
# Complete trat data table:
##head(dat.trait)


# Diagnostic Examinations and messages:
#**************************************
# MESSAGE:
# Number of samples without fish:
##as.matrix(colnames(dat.trait))
a <- nrow(dat.trait[dat.trait$N.sn==0 &  dat.trait$NN.sn==0,])
if(a>0){
message("Your raw data table contains ",a," sample(s) with no fish.") }
# Number of sample with only non-native fish:
b <- nrow(dat.trait[dat.trait$N.sn==0 &  dat.trait$NN.sn>0,])
if(b>0){
message("Your raw data table contains ",b," sample(s) with non-native fish only.") }
rm(a,b)
#++++++++++++++++++++++++++++++++++++++++++++++++++



#   *** III. Computing HMMFI and EQR and EQC ***
#++++++++++++++++++++++++++++++++++++++++++++++++++
# Cycle to assess samples one by one:
#************************************
HMMFI_sample <- NULL # A minta HMMFI indexértéke.
EQR_sample   <- NULL # A minta EQR értéke.
EQC_sample   <- NULL # A minta EQR érték alapján megállapított minőségi osztálya.
for(i in 1:nrow(dat.trait)){
    #----------------------------------------------
    # Set type:
    typ <- paste("typ",dat.trait$type[i], sep="")
    #----------------------------------------------
    # Typespecific argument values:
    metrics <- METRICS[[typ]]
    scrit   <- SCRIT[[typ]]
    scores  <- SCORES[[typ]]
    w       <- WEIGHTS[[typ]]
    # Ellenőrzés: A metrikák sorrendje, nevei egyeznek-e:
    ##all(names(scrit) ==metrics) # OK
    ##all(names(scores)==metrics) # OK
    ##all(names(w)     ==metrics) # OK
    #----------------------------------------------
    # HMMFI_min and HMMFI_max:
    ##all(names(unlist(w))==names(scores))
    HMMFI_min <- sum(sapply(X=scores, FUN=min) * unlist(w))
    HMMFI_min
    HMMFI_max <- sum(sapply(X=scores, FUN=max) * unlist(w))
    HMMFI_max
    #----------------------------------------------
    # Compute HMMFI:
    # If sample has no fish HMMFI should be NA:
    if(dat.trait$N.sn[i]==0 & dat.trait$NN.sn[i]==0){
        HMMFI_sample[i] <-NA }
    # If sample contains fish HMMFI computed from scores:
    if(dat.trait$N.sn[i]>0 | dat.trait$NN.sn[i]>0){
        # Nested cycle to score the sample:
        score_values  <- NULL # Scores of the sample
        for(j in 1:length(metrics)) { # Adott felmérésen minden metrikával végzi ezt:
           # Adott j metrika értékének kiválasztása a felmérési adatokból:
           m <- dat.trait[i,colnames(dat.trait) %in% metrics[j]]
           ##m
           # A metrikaérték függvényében való pontozás (scoring):
           s <- cut(m, include.lowest = T, right = T,
                    breaks=scrit[[ metrics[j] ]],
                    labels=scores[[ metrics[j] ]])
           score_values[j] <- as.numeric(as.character.factor(s))
        } # End of for cycle j
        ##score_values
        ### Ellenőrzés:
        ##all(names(unlist(w))==metrics) # OK

        HMMFI_sample[i] <- sum(score_values * unlist(w))
    } # End of if
    #----------------------------------------------
    # Compute EQR:
    if(dat.trait$N.sn[i]==0 & dat.trait$NN.sn[i]==0){
        EQR_sample[i] <- 0 }
    if(dat.trait$N.sn[i]>0 | dat.trait$NN.sn[i]>0){
        EQR_sample[i]   <- eqr(HMMFI_sample[i], HMMFI_min, HMMFI_max) }
    rm(HMMFI_min, HMMFI_max)
    #----------------------------------------------
    # Classifying sample into ECQ:
    EQC_sample[i] <- as.character.factor(
                        cut(EQR_sample[i], include.lowest = T, right = T,
                            breaks=c(0, 0.2, 0.4, 0.6, 0.8, 1),
                            labels=c("bad","poor","moderate","good","high")) ) #labels=c(1, 2, 3, 4, 5)
    #----------------------------------------------
} # End of for cycle i
##HMMFI_sample
##EQR_sample
##EQC_sample
#++++++++++++++++++++++++++++++++++++++++++++++++++


#        *** IV. Return results ***
#++++++++++++++++++++++++++++++++++++++++++++++++++
# Result object: data.frame:
#***************************
res <- cbind(dat[,1:2], HMMFI=HMMFI_sample, EQR=EQR_sample,
             EQC=EQC_sample, stringsAsFactors=FALSE)
##str(res)


# Write result object into WD:
#*****************************
# Date and time of run:
datetime <- format(Sys.Date(),"%Y%m%d")
write.table(res, paste("RESULTS_",datetime,".csv", sep=""), sep=";", row.names=FALSE, fileEncoding="UTF-8")
# MESSAGE:
{cat("\nResults were written into your working directory in the file '",paste("RESULTS_",datetime,".csv", sep=""),"'.\n", sep="")
cat("\nPlease, cite it:\n")
cat("\nSály, P., Erős, T. (2016): Ecological assessment of running waters
  in Hungary: compilation of biotic indices based on fish [Vízfolyások ökológiai
  állapotminősítése halakkal: minősítési indexek kidolgozása].
  Pisces Hungarici 10:15--45. (In Hungarian with an English abstract)\n")
cat("\n*** End of program ***   <')))>< \n") }
#++++++++++++++++++++++++++++++++++++++++++++++++++
#======================================================================
# *** End of Code ***
